#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/12/18 12:50
Desc: cost配置文件
"""
url = "https://www.expatistan.com/cost-of-living/index"
name_url_map = {
    "europe": "/cost-of-living/index/europe",
    "north-america": "/cost-of-living/index/north-america",
    "latin-america": "/cost-of-living/index/latin-america",
    "asia": "/cost-of-living/index/asia",
    "middle-east": "/cost-of-living/index/middle-east",
    "africa": "/cost-of-living/index/africa",
    "oceania": "/cost-of-living/index/oceania",
    "world": "/cost-of-living/index",
}
